#RUN: %fish %s
# Check we can handle more than 64 threads (if not this HANGS!):
true (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true  (true )))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
